function [f,A,b,equalityvector,integervector]=inequality_endogLP(M,p,assignquant,e)
% This function formulates the Mixed Integer Linear Programming problem
% to recover "delta" in the benchmark case of Inequality aversion.
%
% Inputs: 
% M number of group members (restricted to 2 in this code)
% p price vector
% assignquant vector containing observed quantities (first T rows: member 1
% / second set of T rows: member 2)
% afriat index e (= 1)
%
% Outputs:
% objective function f (trivial)
% A matrix of LHS
% b matrix of RHS
% equalityvector: equality or inequality constraint
% integervector: integer or continuous variable

%% ***CHECK INPUT PARAMETERS***
T=size(p,1);
N=size(p,2);

% Define smallnumber: used for a<b restrictions (modelled as
% a<=b-smallnumber)
global smallnumber
smallnumber=0.0001;

global largenumber % larger than budgets
largenumber=100;

%% ***Define dimensions of the problem: decision variables and nr of constraints***
% Define decision variables
nrx=M*T*T; % binary pref. relations
nrshadow=M*T*N; % shadow prices for egoism
nrineqaversion=M*T*N; % shadow prices ass. with ineq. aversion
nrdelta=1;
nrvars=nrx+nrshadow+nrineqaversion+nrdelta;

% Define for each variable whether it is binary integer ('b'), general integer ('i'),
% or continuous ('c')
intx=repmat('b',M*T*T,1);
intshadow=repmat('c',M*T*N,1);
intineqaversion=repmat('c',M*T*N,1);
intdelta=repmat('c',nrdelta,1);
integervector=[intx;intshadow;intineqaversion;intdelta]; % vector of length nrvars. Contains 'b' for binary, 'i' for general integer and 'c' for continuous

d=zeros(T,N);

%% create d function
for t=1:T
    for n=1:N
        if assignquant(t,n)==assignquant(T+t,n)
            d(t,n)=0;%+24
        else
            d(t,n)=0-max(assignquant(t,n)-assignquant(T+t,n),assignquant(T+t,n)-assignquant(t,n));%+24
        end
        
    end
end

%% FORMULATE BIP PROBLEM
% Make objective function
f=zeros(nrvars,1); % all-zero vector
f(end,1)=1;

A=zeros(nrvars,M*T*T+M*T*(T-1)*(T-2)+M*T*T+3*T*N);
b=zeros(1,M*T*T+M*T*(T-1)*(T-2)+M*T*T+3*T*N);

equalityvector=zeros(M*T*T+M*T*(T-1)*(T-2)+M*T*T+3*T*N,1);

%% Condition set 1: GARP
r=1;

% GARP opening condition
for m=1:M
    for t=1:T
        for v=1:T
            
            A((m-1)*T*T+(t-1)*T+v,r)=-100;
            for n=1:N
                A(nrx+(m-1)*T*N+(t-1)*N+n,r)=-assignquant((m-1)*T+v,n)+e*assignquant((m-1)*T+t,n);
            end
            for n=1:N
                A(nrx+M*T*N+(m-1)*T*N+(t-1)*N+n,r)=-d(v,n)+e*d(t,n);
            end
            b(1,r)=-smallnumber;
            equalityvector(r,1)=0;
            r=r+1;
                
        end
    end
end

% Transitivity
for m=1:M
    for s=1:T
        for u=[1:s-1,s+1:T]
            for t=[1:u-1,u+1:T]
                if not(isequal(s,t))
                    A((m-1)*T*T+(s-1)*T+u,r)=1;
                    A((m-1)*T*T+(u-1)*T+t,r)=1;
                    A((m-1)*T*T+(s-1)*T+t,r)=-1;
                    b(1,r)=1;
                    equalityvector(r,1)=0;
                    r=r+1;
                end % if
            end % for t
        end % for u
    end % for s
end

% GARP closing condition
for m=1:M
    for t=1:T
        for v=1:T
            
            A((m-1)*T*T+(t-1)*T+v,r)=100;
            for n=1:N
                A(nrx+(m-1)*T*N+(v-1)*N+n,r)=-assignquant((m-1)*T+t,n)+e*assignquant((m-1)*T+v,n);
            end
            for n=1:N
                A(nrx+M*T*N+(m-1)*T*N+(v-1)*N+n,r)=-d(t,n)+e*d(v,n);
            end
            b(1,r)=100;
            equalityvector(r,1)=0;
            r=r+1;
            
        end
    end
end

%% Condition set 2: FOCs

for t=1:T
    for n=1:N
        if (assignquant(t,n)>assignquant(T+t,n))% || assignquant(t,n)*assignquant(T+t,n)==0) %assignquant(t,n)>assignquant(T+t,n) %|| assignquant(T+t,n)==0 %&& assignquant(t,n)~=0
             
            % condition set 1
            A(nrx+T*N+(t-1)*N+n,r)=1; %z22
            A(nrx+M*T*N+(t-1)*N+n,r)=1; %z1d
            A(nrx+M*T*N+T*N+(t-1)*N+n,r)=1; %z2d
            b(1,r)=p(t,n);
            if assignquant(T+t,n)>0 
                equalityvector(r,1)=1;
            else
                equalityvector(r,1)=0;
            end
            r=r+1; 
            
            % condition set 2
            A(nrx+(t-1)*N+n,r)=1; %z11
            A(nrx+M*T*N+(t-1)*N+n,r)=-1; %z1d
            A(nrx+M*T*N+T*N+(t-1)*N+n,r)=-1; %z2d
            b(1,r)=p(t,n);
            if assignquant(t,n)>0 
                equalityvector(r,1)=1;
            else
                equalityvector(r,1)=0;
            end
            r=r+1; 
            
        elseif (assignquant(t,n)<assignquant(T+t,n))%|| assignquant(t,n)*assignquant(T+t,n)==0)%assignquant(T+t,n)>assignquant(t,n) %|| assignquant(t,n)==0

            % condition set 1
            A(nrx+(t-1)*N+n,r)=1; %z11
            A(nrx+M*T*N+(t-1)*N+n,r)=1; %z1d
            A(nrx+M*T*N+T*N+(t-1)*N+n,r)=1; %z2d
            b(1,r)=p(t,n);
            if assignquant(t,n)>0 
                equalityvector(r,1)=1;
            else
                equalityvector(r,1)=0;
            end
            r=r+1; 
            
            % condition set 2
            A(nrx+T*N+(t-1)*N+n,r)=1; %z22
            A(nrx+M*T*N+(t-1)*N+n,r)=-1; %z1d
            A(nrx+M*T*N+T*N+(t-1)*N+n,r)=-1; %z2d
            b(1,r)=p(t,n);
            if assignquant(T+t,n)>0 
                equalityvector(r,1)=1;
            else
                equalityvector(r,1)=0;
            end
            r=r+1; 
            
        elseif assignquant(t,n)==assignquant(T+t,n)% || assignquant(t,n)*assignquant(T+t,n)==0) %assignquant(t,n)>assignquant(T+t,n) %|| assignquant(T+t,n)==0 %&& assignquant(t,n)~=0
             
            % condition set 1
            A(nrx+T*N+(t-1)*N+n,r)=1; %z22
            A(nrx+M*T*N+(t-1)*N+n,r)=-1; %z1d
            A(nrx+M*T*N+T*N+(t-1)*N+n,r)=-1; %z2d
            b(1,r)=p(t,n);
            if assignquant(T+t,n)>0 
                equalityvector(r,1)=1;
            else
                equalityvector(r,1)=0;
            end
            r=r+1; 
            
            % condition set 2
            A(nrx+(t-1)*N+n,r)=1; %z11
            A(nrx+M*T*N+(t-1)*N+n,r)=-1; %z1d
            A(nrx+M*T*N+T*N+(t-1)*N+n,r)=-1; %z2d
            b(1,r)=p(t,n);
            if assignquant(t,n)>0 
                equalityvector(r,1)=1;
            else
                equalityvector(r,1)=0;
            end
            r=r+1;                 

        end
    end
end

%% Condition set 4: delta restrictions
for t=1:T
    for n=1:N
        A(nrx+M*T*N+(t-1)*N+n,r)=1;
        A(nrx+M*T*N+T*N+(t-1)*N+n,r)=1;
        A(nrx+M*T*N+M*T*N+1,r)=-p(t,n);
        b(1,r)=0;
        equalityvector(r,1)=0;
        r=r+1;
    end
end

A=A';
b=b';
return