% This program generates the numerical results in Table 1.
% Paste the commented code below in your command window.
% It calls function Simulation.
% Input: theta = test.
% Output: vector estimate, containing "pi" or "delta".

%% for Altruism (spec 5): paste this code in command window
% clear;clc;
% estimate=99*ones(11,1);
% for i=0.39:0.01:0.49
%     ORP=runsimulation(i,5);
%     estimate(round(i*100-38),1)=ORP;
% end

%% for Inequality aversion (spec 6): paste this code in command window
% clear;clc;
% estimate=99*ones(11,1);
% for i=0.39:0.002:0.41
%     ORP=runsimulation(i,6);
%     estimate(round(i*500-194),1)=ORP;
% end

%%
function ORP=runsimulation(test,spec)   
N = 9;
e = 1;
pall=[8 4 1;8 3 2;9 3 1;1 8 4;2 8 3;1 9 3;4 1 8;3 2 8;3 1 9];

%% select appropriate specification of preferences
if spec == 5
    
    mu = @(t) (t(1)/(2.5*(t(2)+t(3))-0.999))^3;
    alpha = [1-2*test 0 0 0 0 0 2*test 0 0];
    beta = [test 0.9*(1-test) 0 0 0.1*(1-test) 0 0 0 0]; 

elseif spec == 6
    
    mu = @(t) (t(1)/(2.5*(t(2)+t(3))-0.999))^3;
    alpha = [1-2*test 0 0 0 0 0 2*test 0 0];
    beta = [0 0.9*(1-test) test 0 0.1*(1-test) 0 0 0 0]; 

end
    
%% Simulate demand
    demandfun = demandspec([alpha beta],mu,spec);   
           
    % Generate data using demand equations
    p = zeros(N,3);
    y = zeros(N,1);
    q = zeros(N,6);
    assignquant = zeros(2*N,3);
    
    for j=1:N
        p(j,:) = pall(j,:);
        y(j) = 24;
        
        q(j,:) = demandfun([p(j,:),y(j)]);
        assignquant(j,:) = [q(j,1) q(j,2) q(j,3)];
        assignquant(N+j,:) = [q(j,4) q(j,5) q(j,6)];
    end  
    
    if spec==5
        
        ORP = altruism_endog(2,p,assignquant,e); % pi
        
    elseif spec==6
        
        ORP = inequality_endog(2,p,assignquant,e); % delta
        
    end
end

function demandfun = demandspec(param,mu,spec)
    alpha = param(1:9);
    beta = param(10:18);
                    
    demandfun = @q_function;
    
    function q = q_function(t)
        q = zeros(6,1);
        while repmat([t(1) t(2) t(3)],1,2)*q>t(4)+0.5 || repmat([t(1) t(2) t(3)],1,2)*q<t(4)-0.5
            maxU = -Inf;
            for start=1:10
                  if spec ~=6
                      
                    [maxU_i,demand_i] = simulatedDemand_CES(t);
                    if maxU_i>=maxU
                        maxU=maxU_i;
                        q=demand_i;
                    end
                    
                  elseif spec == 6
                      
                      for n1all=1:2:3
                        [maxU_i,demand_i] = simulatedDemand_CES_spec6(t,n1all-2);
                        if maxU_i>=maxU
                            maxU=maxU_i;
                            q=demand_i;
                        end
                      end
                      
                  end
            end
        end

        q=q';
    end 

    function [maxU,x] = simulatedDemand_CES(t)
        alpha = param(1:9);
        beta = param(10:18);
        
        p1=t(1);
        p2=t(2);
        p3=t(3);
        y=t(4);
        
        mu([p1 p2 p3]);
        
        function f = myfun(x)
            rho=0.85;
            f = (1-mu([p1 p2 p3]))*(alpha(1)*x(1)^rho+alpha(2)*x(4)^rho+...
                alpha(4)*x(2)^rho+alpha(5)*x(5)^rho+...
                alpha(7)*x(3)^rho+alpha(8)*x(6)^rho+...
                alpha(3)*(24-abs(x(1)-x(4)))^rho+...
                alpha(6)*(24-abs(x(2)-x(5)))^rho+...
                alpha(9)*(24-abs(x(3)-x(6)))^rho)^(1/rho)+...
                mu([p1 p2 p3])*(beta(1)*x(1)^rho+beta(2)*x(4)^rho+...
                beta(4)*x(2)^rho+beta(5)*x(5)^rho+...
                beta(7)*x(3)^rho+beta(8)*x(6)^rho+...
                beta(3)*(24-abs(x(1)-x(4)))^rho+...
                beta(6)*(24-abs(x(2)-x(5)))^rho+...
                beta(9)*(24-abs(x(3)-x(6)))^rho)^(1/rho);
             f = -f;
        end
        
        Aeq=zeros(1,6);
        beq=zeros(1,1);
        
        r=1;
        Aeq(r,:)=[p1 p2 p3 p1 p2 p3];
        beq(r,1)=y;
        
        randnr=rand(6,1);
        x0=randnr.*24./repmat([p1;p2;p3],2,1);

        options=optimset('Display','off');
        [x,fval]=fmincon(@myfun,x0,[],[],Aeq,beq,zeros(6,1),y*ones(6,1),[],options);
        maxU=-fval;       
        
        x=round(x*10000)/10000;
    end

    function [maxU,x] = simulatedDemand_CES_spec6(t,n1)
        alpha = param(1:9);
        beta = param(10:18);
        
        p1=t(1);
        p2=t(2);
        p3=t(3);
        y=t(4);
        
        mu([p1 p2 p3]);
        
        function f = myfun(x)
            rho=0.85;
            f = (1-mu([p1 p2 p3]))*(alpha(1)*x(1)^rho+alpha(2)*x(4)^rho+...
                alpha(4)*x(2)^rho+alpha(5)*x(5)^rho+...
                alpha(7)*x(3)^rho+alpha(8)*x(6)^rho+...
                alpha(3)*(24-n1*(x(1)-x(4)))^rho+...
                alpha(6)*(24+(x(2)-x(5)))^rho+...
                alpha(9)*(24-(x(3)-x(6)))^rho)^(1/rho)+...
                mu([p1 p2 p3])*(beta(1)*x(1)^rho+beta(2)*x(4)^rho+...
                beta(4)*x(2)^rho+beta(5)*x(5)^rho+...
                beta(7)*x(3)^rho+beta(8)*x(6)^rho+...
                beta(3)*(24-n1*(x(1)-x(4)))^rho+...
                beta(6)*(24+(x(2)-x(5)))^rho+...
                beta(9)*(24-(x(3)-x(6)))^rho)^(1/rho);
            f = -f;
        end
        
        Aeq=zeros(1,6);
        beq=zeros(1,1);
        
        A=zeros(3,6);
        b=zeros(3,1);
        
        A(1,:)=[-n1 0 0 n1 0 0];
        A(2,:)=[0 1 0 0 -1 0];
        A(3,:)=[0 0 -1 0 0 1];
        
        r=1;
        Aeq(r,:)=[p1 p2 p3 p1 p2 p3];
        beq(r,1)=y;
        
        randnr=rand(6,1);
        x0=randnr.*24./repmat([p1;p2;p3],2,1);

        options=optimset('Display','off');
        [x,fval]=fmincon(@myfun,x0,A,b,Aeq,beq,zeros(6,1),y*ones(6,1),[],options);
        maxU=-fval;       
        
        x=round(x*10000)/10000;
    end
end